import configparser
import os

import boto3

from hacksaws import _configs


def logout(context: _configs.Context):
    """Cleans up a login session for the aws user."""
    if not context.storage_path.exists():
        return

    credentials = configparser.ConfigParser()
    credentials.read(context.credentials_path)

    storage = configparser.ConfigParser()
    storage.read(context.storage_path)

    # Copy stored values.
    credentials[context.profile] = storage[context.profile]

    with open(context.credentials_path, "w+") as fp:
        credentials.write(fp)

    os.remove(context.storage_path)


def login(context: _configs.Context):
    """Executes a login process."""
    configs = configparser.ConfigParser()
    configs.read(context.config_path)
    profile_configs = configs[f"profile {context.profile}"]

    session = boto3.Session(profile_name=context.profile)
    client = session.client("sts")
    response = client.get_session_token(
        DurationSeconds=context.args.lifespan,
        SerialNumber=profile_configs["mfa_serial"],
        TokenCode=context.args.mfa_code,
    )
    creds = response["Credentials"]

    credentials = configparser.ConfigParser()
    credentials.read(context.credentials_path)

    storage = configparser.ConfigParser()
    storage[context.profile] = credentials[context.profile]

    with open(context.storage_path, "w+") as fp:
        storage.write(fp)

    credentials[context.profile] = {
        "aws_access_key_id": creds["AccessKeyId"],
        "aws_secret_access_key": creds["SecretAccessKey"],
        "aws_session_token": creds["SessionToken"],
    }

    with open(context.credentials_path, "w+") as fp:
        credentials.write(fp)



import argparse
import dataclasses
import pathlib
import typing

import boto3


@dataclasses.dataclass(frozen=True)
class Context:
    """Data structure for cli executions."""

    #: Parsed command line arguments for the invocation.
    args: argparse.Namespace

    @property
    def profile(self) -> str:
        """AWS profile name associated with this command execution."""
        return self.get("profile")

    @property
    def aws_directory(self) -> pathlib.Path:
        """
        AWS credentials directory where the credentials and config file
        reside.
        """
        if name := self.get("aws_account_name"):
            value = f"~/.aws-{name}"
        else:
            value = self.get("directory") or "~/.aws"
        return pathlib.Path(value).expanduser().absolute()

    @property
    def credentials_path(self) -> pathlib.Path:
        """Path to the AWS credentials file."""
        return self.aws_directory.joinpath("credentials")

    @property
    def config_path(self) -> pathlib.Path:
        """Path to the AWS config file."""
        return self.aws_directory.joinpath("config")

    @property
    def storage_path(self) -> pathlib.Path:
        """Path where static credentials will be saved while logged in."""
        return self.aws_directory.joinpath(f"{self.profile}.store.credentials")

    def get(self, key: str, default_value: typing.Any = None) -> typing.Any:
        """Retrieves the value from the args object."""
        return getattr(self.args, key, default_value)


@dataclasses.dataclass(frozen=True)
class AwsAccount:
    """Data structure representing basic account ID information."""

    #: Response object returned from an sts.get_caller_identity request.
    identity_response: dict
    region_name: str
    ecr_additional_regions: typing.Tuple[str, ...]

    @property
    def id(self) -> typing.Optional[str]:
        return self.identity_response.get("Account")

    @property
    def user_arn(self) -> typing.Optional[str]:
        return self.identity_response.get("Arn")

    @property
    def user_id(self) -> typing.Optional[str]:
        return self.identity_response.get("UserId")

    @property
    def ecr_registries(self) -> typing.List[str]:
        region_names = [self.region_name] + list(self.ecr_additional_regions)
        return [f"{self.id}.dkr.ecr.{rn}.amazonaws.com" for rn in region_names]

    @classmethod
    def from_context(cls, context: "Context") -> "AwsAccount":
        session = boto3.Session(profile_name=context.profile)
        return cls(
            identity_response=session.client("sts").get_caller_identity(),
            region_name=session.region_name or "us-east-1",
            ecr_additional_regions=tuple(set(context.args.ecr_region or [])),
        )


@dataclasses.dataclass(frozen=True)
class Result:
    """Data structure containing the result of a command execution."""

    code: str
    message: str

    def echo(self) -> "Result":
        print(f"\n{self.message}\n")
        return self



import argparse
import os
import typing

from hacksaws import _aws
from hacksaws import _configs
from hacksaws import _ecr


def _create_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="hacksaws",
        description="CLI for dynamic credential login management in AWS.",
        allow_abbrev=False,
    )
    type_subparsers = parser.add_subparsers(dest="access_type")

    mfa_type_parser = type_subparsers.add_parser("mfa")
    subparsers = mfa_type_parser.add_subparsers(dest="action")

    login_parser = subparsers.add_parser("login", aliases=["in"])
    login_parser.add_argument("profile")
    login_parser.add_argument("mfa_code")
    login_parser.add_argument("-l", "--lifespan", type=int, default=43200)

    logout_parser = subparsers.add_parser("logout", aliases=["out"])
    logout_parser.add_argument("profile")

    for p in [login_parser, logout_parser]:
        p.add_argument("--ecr", action="store_true")
        p.add_argument("--ecr-region", action="append")
        p.add_argument(
            "-d",
            "--dir",
            "--directory",
            dest="directory",
            default=os.path.expanduser("~/.aws"),
        )
        p.add_argument(
            "-n",
            "--name",
            "--account-name",
            dest="aws_account_name",
        )

    return parser


def _print_help(command: typing.List[str] = None):
    """Prints command help without exiting the program."""
    try:
        _create_parser().parse_args((command or []) + ["--help"])
    except SystemExit:
        pass


def _run_mfa(context: _configs.Context) -> _configs.Result:
    """Executes an action for an mfa access type."""
    action = context.args.action
    if not action:
        _print_help(["mfa"])
        return _configs.Result(
            code="MFA_HELP",
            message="Not enough arguments specified for the mfa command.",
        )

    # Configure the session credential file locations based on CLI input.
    os.environ["AWS_SHARED_CREDENTIALS_FILE"] = str(context.credentials_path)
    os.environ["AWS_CONFIG_FILE"] = str(context.config_path)

    # Log out first no matter what action is taking place.
    _aws.logout(context)

    # Get account data
    aws_account = _configs.AwsAccount.from_context(context)

    if context.args.ecr:
        _ecr.logout(aws_account)

    if action in ("login", "in"):
        _aws.login(context)
        if context.args.ecr:
            _ecr.login(context, aws_account)

        return _configs.Result(
            code="MFA_LOGIN",
            message=f"Logged into profile {context.profile}",
        )

    # Only other available action is logging out.
    return _configs.Result(
        code="MFA_LOGOUT",
        message=f"Logged out of profile {context.profile}",
    )


def console_main(arguments: typing.List[str] = None) -> _configs.Result:
    """Entrypoint for command line invocations."""
    try:
        parser = _create_parser()
        context = _configs.Context(args=parser.parse_args(arguments))
    except SystemExit:
        return _configs.Result("HELP", "Displayed command help.")

    access_type = context.args.access_type
    if not access_type:
        _print_help()
        result = _configs.Result(
            code="ACCESS_TYPE_HELP",
            message="Not enough arguments.",
        )
    else:
        result = _run_mfa(context)

    return result.echo()


# if __name__ == '__main__':
#     console_main(['mfa', 'login', '--name=scout', 'billing-ro', '123456'])



import base64
import datetime
import subprocess

import boto3

from hacksaws import _configs


def _do_login(context: _configs.Context, registry: str):
    """Carry out the loging process for a region-specific registry."""
    print(f"[STARTED]: Logging into {registry}", flush=True)
    parts = registry.split(".")
    account_id = parts[0]
    region_name = parts[3]

    session = boto3.Session(profile_name=context.profile, region_name=region_name)
    response = (
        session.client("ecr")
        .get_authorization_token(registryIds=[account_id])
        .get("authorizationData", {})[0]
    )

    user, password = (
        base64.b64decode(response["authorizationToken"].encode()).decode().split(":")
    )

    cmd = [
        "docker",
        "login",
        f"--username={user}",
        "--password-stdin",
        registry,
    ]
    subprocess.run(cmd, input=password.encode(), check=True)

    expires_at: datetime.datetime = (
        response["expiresAt"].astimezone(datetime.timezone.utc).replace(tzinfo=None)
    )

    delta: datetime.timedelta = expires_at - datetime.datetime.utcnow()
    hours = int(round(delta.total_seconds() / 3600))
    print(f"[SUCCESS]: Login session will expire in {hours} hours", flush=True)


def login(context: _configs.Context, aws_account: _configs.AwsAccount):
    """Logs into the AWS ECR registry in the given account."""
    for registry in aws_account.ecr_registries:
        _do_login(context, registry)


def logout(aws_account: _configs.AwsAccount):
    """Logs out of the registry for the given AWS account."""
    for registry in aws_account.ecr_registries:
        cmd = ["docker", "logout", registry]
        subprocess.run(cmd, check=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hacksaws']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.16.0,<2.0.0']

entry_points = \
{'console_scripts': ['hacksaws = hacksaws:main']}

setup_kwargs = {
    'name': 'hacksaws',
    'version': '0.2.0',
    'description': 'A command line utility for AWS profiles using dynamic authentication methods such as MFA. ',
    'long_description': '# hacksaws\n\n[![PyPI version](https://badge.fury.io/py/hacksaws.svg)](https://badge.fury.io/py/hacksaws)\n[![build status](https://gitlab.com/rocket-boosters/hacksaws/badges/main/pipeline.svg)](https://gitlab.com/rocket-boosters/hacksaws/commits/main)\n[![coverage report](https://gitlab.com/rocket-boosters/hacksaws/badges/main/coverage.svg)](https://gitlab.com/rocket-boosters/hacksaws/commits/main)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Code style: flake8](https://img.shields.io/badge/code%20style-flake8-white)](https://gitlab.com/pycqa/flake8)\n[![Code style: mypy](https://img.shields.io/badge/code%20style-mypy-white)](http://mypy-lang.org/)\n[![PyPI - License](https://img.shields.io/pypi/l/hacksaws)](https://pypi.org/project/hacksaws/)\n\nA command line utility for AWS profiles using dynamic authentication \nmethods such as MFA. The CLI allows for dynamic logins to update\nthe credentials for an AWS profile temporarily, while storing the\nlong-term access key and secret in a backup file until the next\nlogin or logout call is made. That way dynamic logins can be used while\nstill maintaining the same functional credential interface as\nnon-dynamic credentials.\n\nAt this time only MFA-based dynamic logins are supported, but SSO\nand others will be added in the future.\n\n## Usage\n\nTo login with MFA, execute the command:\n\n```shell script\n$ hacksaws mfa login <PROFILE_NAME> <MFA_CODE>\n```\n\nThere is a `--lifespan` flag that can be appended here to adjust\nthe amount of time the profile login is valid for before it expires.\nThe default is 12 hours (`--lifetime=43200` seconds), but that can\nbe adjusted to a maximum of 24 hours if the profile login allows\nauthentication lifespans of that length.\n\nECR logins with docker can also be handled with the command by adding the `--ecr`\nflag. This will the local docker environment into ECR in the default AWS region for\nthe specified profile.\n\n```shell script\n$ hacksaws mfa login <PROFILE_NAME> <MFA_CODE> --ecr\n```\n\nIt is also possible to login to ECR in multiple regions with the `--ecr-region` flag.\n\n```shell script\n$ hacksaws mfa login <PROFILE_NAME> <MFA_CODE> \\\n    --ecr \\\n    --ecr-region=eu-central-1 \\\n    --ecr-region=us-west-2 \\\n    --ecr-region=ca-central-1\n```\n\nECR will always log into the AWS default region. The `--ecr-region` flag allows for\nadding additional regions to the login command.\n\nThen to log out:\n\n```shell script\n$ hacksaws mfa logout <PROFILE_NAME>\n```\n\nIt is possible to log in and out of ECR for the account with that\nprofile as well by including the `--ecr` flag in the login call.\n\nAlternate directories for the AWS credentials directory can be\nspecified with the `--directory` flag. \n\nAnd for separated AWS credentials directories in the home directory\nthat follow the pattern `~/.aws-<NAME>`, a `--name` flag can be\nspecified to use that directory instead of the default `~/.aws`\ndirectory. This is a useful pattern for separating profiles by\naccount in cases where one has multiple account credentials.\n\n## Requiring MFA\n\nHere\'s an example policy that allows a user to manage their own\nuser account settings while requiring MFA.\n\n```json\n{\n  "Version": "2012-10-17",\n  "Statement": [\n    {\n      "Sid": "ViewAccountInfo",\n      "Effect": "Allow",\n      "Action": [\n        "iam:ListUsers",\n        "iam:ListAccount*",\n        "iam:ListMFADevices",\n        "iam:GetAccountPasswordPolicy",\n        "iam:GetAccountSummary"\n      ],\n      "Resource": "*"\n    },\n    {\n      "Sid": "ViewAndManageTheirUser",\n      "Effect": "Allow",\n      "Action": [\n        "iam:*LoginProfile",\n        "iam:*AccessKey*",\n        "iam:*SSHPublicKey*",\n        "iam:*SigningCertificate*",\n        "iam:*ServiceSpecificCredential*",\n        "iam:GetUser",\n        "iam:ChangePassword"\n      ],\n      "Resource": "arn:aws:iam::*:user/${aws:username}"\n    },\n    {\n      "Sid": "ManageTheirOwnMFA",\n      "Effect": "Allow",\n      "Action": [\n        "iam:CreateVirtualMFADevice",\n        "iam:DeactivateMFADevice",\n        "iam:DeleteVirtualMFADevice",\n        "iam:EnableMFADevice",\n        "iam:ListMFADevices",\n        "iam:ListVirtualMFADevices",\n        "iam:ResyncMFADevice"\n      ],\n      "Resource": [\n        "arn:aws:iam::*:mfa/${aws:username}",\n        "arn:aws:iam::*:user/${aws:username}"\n      ]\n    },\n    {\n      "Sid": "DenyAllExceptListedIfNoMFA",\n      "Effect": "Deny",\n      "NotAction": [\n        "iam:ListUsers",\n        "iam:ListMFADevices",\n        "iam:ChangePassword",\n        "iam:GetUser",\n        "iam:CreateVirtualMFADevice",\n        "iam:EnableMFADevice",\n        "iam:ListMFADevices",\n        "iam:ListVirtualMFADevices",\n        "iam:ResyncMFADevice",\n        "sts:GetSessionToken"\n      ],\n      "Resource": "*",\n      "Condition": {\n        "BoolIfExists": {\n          "aws:MultiFactorAuthPresent": "false"\n        }\n      }\n    }\n  ]\n}\n```\n\nControlling password quality and expiration policies is an account-level requirement\nand more details can be found at\n(Setting an account password policy for IAM users)[https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_passwords_account-policy.html]\n\nAdditional Resources:\n\n- [Allows MFA-authenticated IAM users...](https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_aws_my-sec-creds-self-manage-mfa-only.html)\n- [IAM: Allows IAM users to self-manage an MFA device](https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_examples_iam_mfa-selfmanage.html)\n- [Configuring MFA-protected API access](https://docs.amazonaws.cn/en_us/IAM/latest/UserGuide/id_credentials_mfa_configure-api-require.html)\n',
    'author': 'Scott Ernst',
    'author_email': 'swernst@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/rocket-boosters/hacksaws',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)
